@@ -253,21 +253,24 @@ class TypesConverterCursorFactory:
                     bindings: apsw.Bindings | None = None,
                     *,
                     can_cache: bool = True,
-                    prepare_flags: int = 0) -> apsw.Cursor:
+                    prepare_flags: int = 0,
+                    explain: int = -1) -> apsw.Cursor:
             """Executes the statements doing conversions on supplied and returned values
 
             See :meth:`apsw.Cursor.execute` for parameter details"""
             return super().execute(statements,
                                    self.factory.wrap_bindings(bindings),
                                    can_cache=can_cache,
-                                   prepare_flags=prepare_flags)
+                                   prepare_flags=prepare_flags,
+                                   explain=explain)
 
         def executemany(self,
                         statements: str,
                         sequenceofbindings: Sequence[apsw.Bindings],
                         *,
                         can_cache: bool = True,
-                        prepare_flags: int = 0) -> apsw.Cursor:
+                        prepare_flags: int = 0,
+                        explain: int = -1) -> apsw.Cursor:
             """Executes the statements against each item in sequenceofbindings, doing conversions on supplied and returned values
 
             See :meth:`apsw.Cursor.executemany` for parameter details"""
@@ -275,18 +278,25 @@ class TypesConverterCursorFactory:
                 statements,
                 self.factory.wrap_sequence_bindings(sequenceofbindings),  # type: ignore[arg-type]
                 can_cache=can_cache,
-                prepare_flags=prepare_flags)
+                prepare_flags=prepare_flags,
+                explain=explain)
 
 
 def log_sqlite(*, level: int = logging.ERROR) -> None:
     """Send SQLite log messages to :mod:`logging`
 
-    :param level: level to log at (default *logging.ERROR*)
+    :param level: level to log at
+
+    SQLite's `logging <https://www.sqlite.org/errlog.html>`__ has many
+    useful messages, including those that aren't raised as exceptions.
     """
 
     def handler(errcode: int, message: str) -> None:
-        err_str = apsw.mapping_result_codes[errcode & 255]
+        nonlocal level
+        err_str = apsw.mapping_result_codes.get(errcode & 255, str(errcode))
         extra = {"sqlite_code": errcode, "sqlite_code_name": err_str, "sqlite_message": message}
+        if errcode & 0xff == apsw.SQLITE_WARNING:
+            level = min(level, logging.WARNING)
         logging.log(level,
                     "SQLITE_LOG: %s (%d) %s %s",
                     message,
@@ -459,7 +469,8 @@ def dbinfo(db: apsw.Connection,
     Memory databases return `None` for both.
     """
 
-    dbinfo = journalinfo = None
+    dbinfo: DatabaseFileInfo | None = None
+    journalinfo: JournalFileInfo | WALFileInfo | None = None
 
     try:
         ok, header_page = db.read(schema, 0, 0, 128)
@@ -484,7 +495,7 @@ def dbinfo(db: apsw.Connection,
         return {0: "(pending)", 1: "UTF-8", 2: "UTF-16le", 3: "UTF-16be"}.get(v, f"<< INVALID VALUE { v } >>")
 
     if ok:
-        kw = {"filename": db.filename}
+        kw: dict[str, Any] = {"filename": db.filename}
         for name, offset, size, converter in (
             ("header", 0, 16, bytes),
             ("page_size", 16, 2, be_page_size),
@@ -506,7 +517,7 @@ def dbinfo(db: apsw.Connection,
             ("sqlite_version", 96, 4, be_int),
         ):
             b = header_page[offset:offset + size]
-            kw[name] = converter(b)
+            kw[name] = converter(b)  # type: ignore [operator]
         dbinfo = DatabaseFileInfo(**kw)
 
     try:
@@ -515,7 +526,7 @@ def dbinfo(db: apsw.Connection,
         ok = False
 
     if ok:
-        kw = {}
+        kw: dict[str, Any] = {}  # type: ignore [no-redef]
         if db.pragma("journal_mode") == "wal":
             kw["filename"] = db.filename_wal
             for name, offset, size, converter in (
@@ -529,7 +540,7 @@ def dbinfo(db: apsw.Connection,
                 ("checksum_2", 28, 4, be_int),
             ):
                 b = journal_page[offset:offset + size]
-                kw[name] = converter(b)
+                kw[name] = converter(b)  # type: ignore [operator]
             journalinfo = WALFileInfo(**kw)
         else:
             header_valid = lambda b: b == b"\xd9\xd5\x05\xf9\x20\xa1\x63\xd7"
@@ -544,7 +555,7 @@ def dbinfo(db: apsw.Connection,
                 ("page_size", 24, 4, be_int),
             ):
                 b = journal_page[offset:offset + size]
-                kw[name] = converter(b)
+                kw[name] = converter(b)  # type: ignore [operator]
             journalinfo = JournalFileInfo(**kw)
 
     return dbinfo, journalinfo
@@ -1137,6 +1148,13 @@ def make_virtual_module(db: apsw.Connection,
                 self.repr_invalid = module.repr_invalid
                 self.num_columns = len(self.columns)
                 self.access = self.module.column_access
+                col_func = f"_Column_{ self.access.name }"
+                f = getattr(self, col_func, self.Column)
+                if self.repr_invalid:
+                    setattr(self, "Column", self._Column_repr_invalid)
+                    setattr(self, "_Column_get", f)
+                else:
+                    setattr(self, "Column", f)
 
             def Filter(self, idx_num: int, idx_str: str, args: tuple[apsw.SQLiteValue]) -> None:
                 params: dict[str, apsw.SQLiteValue] = self.param_values.copy()
@@ -1159,22 +1177,37 @@ def make_virtual_module(db: apsw.Connection,
                     self.iterating = None
 
             def Column(self, which: int) -> apsw.SQLiteValue:
+                # This is the specification/documentation for the custom
+                # versions which should produce exactly the same output
                 if which >= self.num_columns:
                     return self.hidden_values[which - self.num_columns]
-                v: Any = None
-                if self.access == VTColumnAccess.By_Index:
+                if self.access is VTColumnAccess.By_Index:
                     v = self.current_row[which]
-                elif self.access == VTColumnAccess.By_Name:
+                elif self.access is VTColumnAccess.By_Name:
                     v = self.current_row[self.columns[which]]
-                elif self.access == VTColumnAccess.By_Attr:
+                elif self.access is VTColumnAccess.By_Attr:
                     v = getattr(self.current_row, self.columns[which])
-                if self.repr_invalid:
-                    try:
-                        apsw.format_sql_value(v)
-                    except TypeError:
-                        v = repr(v)
+                if self.repr_invalid and v is not None and not isinstance(v, (int, float, str, bytes)):
+                    v = repr(v)
                 return v  # type: ignore[no-any-return]
 
+            def _Column_repr_invalid(self, which: int) -> apsw.SQLiteValue:
+                v = self._Column_get(which) # type: ignore[attr-defined]
+                return v if v is None or isinstance(v, (int, float, str, bytes)) else repr(v)
+
+            def _Column_By_Attr(self, which: int) -> apsw.SQLiteValue:
+                return getattr(
+                    self.current_row,
+                    self.columns[which]) if which < self.num_columns else self.hidden_values[which - self.num_columns]
+
+            def _Column_By_Name(self, which: int) -> apsw.SQLiteValue:
+                return self.current_row[self.columns[which]] if which < self.num_columns else self.hidden_values[
+                    which - self.num_columns]
+
+            def _Column_By_Index(self, which: int) -> apsw.SQLiteValue:
+                return self.current_row[which] if which < self.num_columns else self.hidden_values[which -
+                                                                                                   self.num_columns]
+
             def Next(self) -> None:
                 try:
                     self.current_row = next(self.iterating)  # type: ignore[arg-type]
@@ -1410,7 +1443,7 @@ def query_info(db: apsw.Connection,
 
     if explain and not res["is_explain"]:
         vdbe: list[VDBEInstruction] = []
-        for row in cur.execute("EXPLAIN " + res["first_query"], bindings):
+        for row in cur.execute(res["first_query"], bindings, explain=1):
             vdbe.append(
                 VDBEInstruction(**dict((v[0][0], v[1]) for v in zip(cur.getdescription(), row) if v[1] is not None)))
         res["explain"] = vdbe
@@ -1419,7 +1452,7 @@ def query_info(db: apsw.Connection,
         subn = "sub"
         byid: Any = {0: {"detail": "QUERY PLAN"}}
 
-        for row in cur.execute("EXPLAIN QUERY PLAN " + res["first_query"], bindings):
+        for row in cur.execute(res["first_query"], bindings, explain=2):
             node = dict((v[0][0], v[1]) for v in zip(cur.getdescription(), row) if v[0][0] != "notused")
             assert len(node) == 3  # catch changes in returned format
             parent: list[str | dict[str, Any]] = byid[node["parent"]]
