@@ -29,7 +29,7 @@ Type Annotations
 Comprehensive `type annotations
 <https://docs.python.org/3/library/typing.html>`__ `are included <https://github.com/rogerbinns/apsw/blob/master/apsw/__init__.pyi>`__,
 and your code can be checked using tools like `mypy
-<http://mypy-lang.org/>`__.  You can refer to the types below for your
+<https://mypy-lang.org/>`__.  You can refer to the types below for your
 annotations (eg as :class:`apsw.SQLiteValue`)
 
 Your source files should include::
@@ -92,8 +92,8 @@ API Reference
 #include "sqlite3.h"
 #endif
 
-#if SQLITE_VERSION_NUMBER < 3042000
-#error Your SQLite version is too old.  It must be at least 3.42
+#if SQLITE_VERSION_NUMBER < 3043000
+#error Your SQLite version is too old.  It must be at least 3.43
 #endif
 
 /* system headers */
@@ -1169,7 +1169,7 @@ static sqlite3_mutex_methods apsw_mutex_methods =
   support the fork system call.
 
   SQLite does not allow the use of database connections across `forked
-  <http://en.wikipedia.org/wiki/Fork_(operating_system)>`__ processes
+  <https://en.wikipedia.org/wiki/Fork_(operating_system)>`__ processes
   (see the `SQLite FAQ Q6 <https://sqlite.org/faq.html#q6>`__).
   (Forking creates a child process that is a duplicate of the parent
   including the state of all data structures in the program.  If you
@@ -1179,17 +1179,17 @@ static sqlite3_mutex_methods apsw_mutex_methods =
 
   One example of how you may end up using fork is if you use the
   `multiprocessing module
-  <http://docs.python.org/library/multiprocessing.html>`__ which uses
+  <https://docs.python.org/library/multiprocessing.html>`__ which uses
   fork to make child processes.
 
   If you do use fork or multiprocessing on a platform that supports
   fork then you **must** ensure database connections and their objects
   (cursors, backup, blobs etc) are not used in the parent process, or
   are all closed before calling fork or starting a `Process
-  <http://docs.python.org/library/multiprocessing.html#process-and-exceptions>`__.
+  <https://docs.python.org/library/multiprocessing.html#process-and-exceptions>`__.
   (Note you must call close to ensure the underlying SQLite objects
   are closed.  It is also a good idea to call `gc.collect(2)
-  <http://docs.python.org/library/gc.html#gc.collect>`__ to ensure
+  <https://docs.python.org/library/gc.html#gc.collect>`__ to ensure
   anything you may have missed is also deallocated.)
 
   Once you run this method, extra checking code is inserted into
@@ -1741,7 +1741,7 @@ apsw_allow_missing_dict_bindings(PyObject *Py_UNUSED(module), PyObject *args, Py
 }
 
 static PyObject *
-apsw_getattr(PyObject *module, PyObject *name)
+apsw_getattr(PyObject *Py_UNUSED(module), PyObject *name)
 {
   PyObject *shellmodule = NULL, *res = NULL;
 #undef PyUnicode_AsUTF8
@@ -1851,7 +1851,7 @@ PyInit_apsw(void)
     goto fail;
   }
 
-  if (PyType_Ready(&ConnectionType) < 0 || PyType_Ready(&APSWCursorType) < 0 || PyType_Ready(&ZeroBlobBindType) < 0 || PyType_Ready(&APSWBlobType) < 0 || PyType_Ready(&APSWVFSType) < 0 || PyType_Ready(&APSWVFSFileType) < 0 || PyType_Ready(&APSWURIFilenameType) < 0 || PyType_Ready(&FunctionCBInfoType) < 0 || PyType_Ready(&APSWBackupType) < 0 || PyType_Ready(&SqliteIndexInfoType) < 0 || PyType_Ready(&apsw_no_change_object) < 0)
+  if (PyType_Ready(&ConnectionType) < 0 || PyType_Ready(&APSWCursorType) < 0 || PyType_Ready(&ZeroBlobBindType) < 0 || PyType_Ready(&APSWBlobType) < 0 || PyType_Ready(&APSWVFSType) < 0 || PyType_Ready(&APSWVFSFileType) < 0 || PyType_Ready(&apswfcntl_pragma_Type) < 0 || PyType_Ready(&APSWURIFilenameType) < 0 || PyType_Ready(&FunctionCBInfoType) < 0 || PyType_Ready(&APSWBackupType) < 0 || PyType_Ready(&SqliteIndexInfoType) < 0 || PyType_Ready(&apsw_no_change_object) < 0)
     goto fail;
 
   /* PyStructSequence_NewType is broken in some Pythons
@@ -1896,6 +1896,7 @@ PyInit_apsw(void)
   ADD(zeroblob, ZeroBlobBindType);
   ADD(VFS, APSWVFSType);
   ADD(VFSFile, APSWVFSFileType);
+  ADD(VFSFcntlPragma, apswfcntl_pragma_Type);
   ADD(URIFilename, APSWURIFilenameType);
   ADD(IndexInfo, SqliteIndexInfoType);
 
@@ -1941,7 +1942,7 @@ PyInit_apsw(void)
       :type: bool
 
       If True then `SQLite amalgamation
-      <https://sqlite.org/cvstrac/wiki?p=TheAmalgamation>`__ is in
+      <https://www.sqlite.org/amalgamation.html>`__ is in
       use (statically compiled into APSW).  Using the amalgamation means
       that SQLite shared libraries are not used and will not affect your
       code.
@@ -1964,7 +1965,7 @@ PyInit_apsw(void)
     :meth:`VTTable.UpdateChangeRow`
   */
 
-  if (PyModule_AddObject(m, "no_change", Py_NewRef(&apsw_no_change_object)))
+  if (PyModule_AddObject(m, "no_change", Py_NewRef((PyObject *)&apsw_no_change_object)))
     goto fail;
 
 #ifdef APSW_TESTFIXTURES
