@@ -6,7 +6,6 @@
 
 /* EXCEPTION TYPES */
 
-static PyObject *APSWException;          /* root exception class */
 static PyObject *ExcThreadingViolation;  /* thread misuse */
 static PyObject *ExcIncomplete;          /* didn't finish previous query */
 static PyObject *ExcBindings;            /* wrong number of bindings */
@@ -26,10 +25,11 @@ static void make_exception(int res, sqlite3 *db);
  work.  We don't overwrite earlier exceptions hence the PyErr_Occurred
  check */
 #define SET_EXC(res, db)                       \
+  do                                           \
   {                                            \
     if (res != SQLITE_OK && !PyErr_Occurred()) \
       make_exception(res, db);                 \
-  }
+  } while (0)
 
 /* A dictionary we store the last error from each thread in.  Used
    thread local storage previously. The key is a PyLong of the thread
@@ -52,22 +52,17 @@ apsw_set_errmsg(const char *msg)
   /* dictionary operations whine if there is an outstanding error */
   PyErr_Fetch(&etype, &eval, &etb);
 
-  if (!tls_errmsg)
-  {
-    tls_errmsg = PyDict_New();
-    if (!tls_errmsg)
-      goto finally;
-  }
+  assert(tls_errmsg);
+
   key = PyLong_FromLong(PyThread_get_thread_ident());
-  if (!key)
-    goto finally;
-  value = PyBytes_FromStringAndSize(msg, strlen(msg));
-  if (!value)
-    goto finally;
+  if (key)
+    value = PyBytes_FromStringAndSize(msg, strlen(msg));
 
-  PyDict_SetItem(tls_errmsg, key, value);
+  if (key && value && 0 == PyDict_SetItem(tls_errmsg, key, value))
+    ;
+  else
+    apsw_write_unraisable(NULL);
 
-finally:
   Py_XDECREF(key);
   Py_XDECREF(value);
   PyErr_Restore(etype, eval, etb);
@@ -84,13 +79,13 @@ apsw_get_errmsg(void)
   assert(tls_errmsg);
 
   key = PyLong_FromLong(PyThread_get_thread_ident());
-  if (!key)
-    goto finally;
-  value = PyDict_GetItem(tls_errmsg, key);
-  if (value)
-    retval = PyBytes_AsString(value);
+  if (key)
+  {
+    value = PyDict_GetItem(tls_errmsg, key);
+    if (value)
+      retval = PyBytes_AsString(value);
+  }
 
-finally:
   Py_XDECREF(key);
   /* value is borrowed */
   return retval;
@@ -101,54 +96,54 @@ static struct
   int code;
   const char *name;
   PyObject *cls;
+  const char *doc;
 } exc_descriptors[] =
     {
         /* Generic Errors */
-        {SQLITE_ERROR, "SQL", NULL},
-        {SQLITE_MISMATCH, "Mismatch", NULL},
-        {SQLITE_NOTFOUND, "NotFound", NULL},
+        {SQLITE_ERROR, "SQL", NULL, SQLError_exc_DOC},
+        {SQLITE_MISMATCH, "Mismatch", NULL, MismatchError_exc_DOC},
+        {SQLITE_NOTFOUND, "NotFound", NULL, NotFoundError_exc_DOC},
 
         /* Internal Errors */
-        {SQLITE_INTERNAL, "Internal", NULL}, /* NOT USED */
-        {SQLITE_PROTOCOL, "Protocol", NULL},
-        {SQLITE_MISUSE, "Misuse", NULL},
-        {SQLITE_RANGE, "Range", NULL},
+        {SQLITE_INTERNAL, "Internal", NULL, InternalError_exc_DOC}, /* NOT USED */
+        {SQLITE_PROTOCOL, "Protocol", NULL, ProtocolError_exc_DOC},
+        {SQLITE_MISUSE, "Misuse", NULL, MisuseError_exc_DOC},
+        {SQLITE_RANGE, "Range", NULL, RangeError_exc_DOC},
 
         /* permissions etc */
-        {SQLITE_PERM, "Permissions", NULL},
-        {SQLITE_READONLY, "ReadOnly", NULL},
-        {SQLITE_CANTOPEN, "CantOpen", NULL},
-        {SQLITE_AUTH, "Auth", NULL},
+        {SQLITE_PERM, "Permissions", NULL, PermissionsError_exc_DOC},
+        {SQLITE_READONLY, "ReadOnly", NULL, ReadOnlyError_exc_DOC},
+        {SQLITE_CANTOPEN, "CantOpen", NULL, CantOpenError_exc_DOC},
+        {SQLITE_AUTH, "Auth", NULL, AuthError_exc_DOC},
 
         /* abort/busy/etc */
-        {SQLITE_ABORT, "Abort", NULL},
-        {SQLITE_BUSY, "Busy", NULL},
-        {SQLITE_LOCKED, "Locked", NULL},
-        {SQLITE_INTERRUPT, "Interrupt", NULL},
-        {SQLITE_SCHEMA, "SchemaChange", NULL},
-        {SQLITE_CONSTRAINT, "Constraint", NULL},
+        {SQLITE_ABORT, "Abort", NULL, AbortError_exc_DOC},
+        {SQLITE_BUSY, "Busy", NULL, BusyError_exc_DOC},
+        {SQLITE_LOCKED, "Locked", NULL, LockedError_exc_DOC},
+        {SQLITE_INTERRUPT, "Interrupt", NULL, InterruptError_exc_DOC},
+        {SQLITE_SCHEMA, "SchemaChange", NULL, SchemaChangeError_exc_DOC},
+        {SQLITE_CONSTRAINT, "Constraint", NULL, ConstraintError_exc_DOC},
 
         /* memory/disk/corrupt etc */
-        {SQLITE_NOMEM, "NoMem", NULL},
-        {SQLITE_IOERR, "IO", NULL},
-        {SQLITE_CORRUPT, "Corrupt", NULL},
-        {SQLITE_FULL, "Full", NULL},
-        {SQLITE_TOOBIG, "TooBig", NULL},
-        {SQLITE_NOLFS, "NoLFS", NULL},
-        {SQLITE_EMPTY, "Empty", NULL},
-        {SQLITE_FORMAT, "Format", NULL},
-        {SQLITE_NOTADB, "NotADB", NULL},
+        {SQLITE_NOMEM, "NoMem", NULL, NoMemError_exc_DOC},
+        {SQLITE_IOERR, "IO", NULL, IOError_exc_DOC},
+        {SQLITE_CORRUPT, "Corrupt", NULL, CorruptError_exc_DOC},
+        {SQLITE_FULL, "Full", NULL, FullError_exc_DOC},
+        {SQLITE_TOOBIG, "TooBig", NULL, TooBigError_exc_DOC},
+        {SQLITE_NOLFS, "NoLFS", NULL, NoLFSError_exc_DOC},
+        {SQLITE_EMPTY, "Empty", NULL, EmptyError_exc_DOC},
+        {SQLITE_FORMAT, "Format", NULL, FormatError_exc_DOC},
+        {SQLITE_NOTADB, "NotADB", NULL, NotADBError_exc_DOC},
 
         {-1, 0, 0}};
 
 /* EXCEPTION CODE */
 
-/* MS compiler is stupid and requires this pulled out */
-
 typedef struct
 {
   PyObject **var;
   const char *name;
+  const char *doc;
 } APSWExceptionMapping;
 
 static int init_exceptions(PyObject *m)
@@ -158,35 +153,34 @@ static int init_exceptions(PyObject *m)
   PyObject *obj;
 
   APSWExceptionMapping apswexceptions[] = {
-      {&ExcThreadingViolation, "ThreadingViolationError"},
-      {&ExcIncomplete, "IncompleteExecutionError"},
-      {&ExcBindings, "BindingsError"},
-      {&ExcComplete, "ExecutionCompleteError"},
-      {&ExcTraceAbort, "ExecTraceAbort"},
-      {&ExcExtensionLoading, "ExtensionLoadingError"},
-      {&ExcConnectionNotClosed, "ConnectionNotClosedError"},
-      {&ExcConnectionClosed, "ConnectionClosedError"},
-      {&ExcCursorClosed, "CursorClosedError"},
-      {&ExcVFSNotImplemented, "VFSNotImplementedError"},
-      {&ExcVFSFileClosed, "VFSFileClosedError"},
-      {&ExcForkingViolation, "ForkingViolationError"}};
+      {&ExcThreadingViolation, "ThreadingViolationError", ThreadingViolationError_exc_DOC},
+      {&ExcIncomplete, "IncompleteExecutionError", IncompleteExecutionError_exc_DOC},
+      {&ExcBindings, "BindingsError", BindingsError_exc_DOC},
+      {&ExcComplete, "ExecutionCompleteError", ExecutionCompleteError_exc_DOC},
+      {&ExcTraceAbort, "ExecTraceAbort", ExecTraceAbort_exc_DOC},
+      {&ExcExtensionLoading, "ExtensionLoadingError", ExtensionLoadingError_exc_DOC},
+      {&ExcConnectionNotClosed, "ConnectionNotClosedError", ConnectionNotClosedError_exc_DOC},
+      {&ExcConnectionClosed, "ConnectionClosedError", ConnectionClosedError_exc_DOC},
+      {&ExcCursorClosed, "CursorClosedError", CursorClosedError_exc_DOC},
+      {&ExcVFSNotImplemented, "VFSNotImplementedError", VFSNotImplementedError_exc_DOC},
+      {&ExcVFSFileClosed, "VFSFileClosedError", VFSFileClosedError_exc_DOC},
+      {&ExcForkingViolation, "ForkingViolationError", ForkingViolationError_exc_DOC}};
 
   /* PyModule_AddObject uses borrowed reference so we incref whatever
      we give to it, so we still have a copy to use */
 
   /* custom ones first */
 
-  APSWException = PyErr_NewException("apsw.Error", NULL, NULL);
+  APSWException = PyErr_NewExceptionWithDoc("apsw.Error", Error_exc_DOC, NULL, NULL);
   if (!APSWException)
     return -1;
-  Py_INCREF(APSWException);
-  if (PyModule_AddObject(m, "Error", (PyObject *)APSWException))
+  if (PyModule_AddObject(m, "Error", Py_NewRef((PyObject *)APSWException)))
     return -1;
 
   for (i = 0; i < sizeof(apswexceptions) / sizeof(apswexceptions[0]); i++)
   {
     PyOS_snprintf(buffy, sizeof(buffy), "apsw.%s", apswexceptions[i].name);
-    *apswexceptions[i].var = PyErr_NewException(buffy, APSWException, NULL);
+    *apswexceptions[i].var = PyErr_NewExceptionWithDoc(buffy, apswexceptions[i].doc, APSWException, NULL);
     if (!*apswexceptions[i].var)
       return -1;
     /* PyModule_AddObject steals the ref, but we don't add a ref for
@@ -200,7 +194,7 @@ static int init_exceptions(PyObject *m)
   for (i = 0; exc_descriptors[i].name; i++)
   {
     PyOS_snprintf(buffy, sizeof(buffy), "apsw.%sError", exc_descriptors[i].name);
-    obj = PyErr_NewException(buffy, APSWException, NULL);
+    obj = PyErr_NewExceptionWithDoc(buffy, exc_descriptors[i].doc, APSWException, NULL);
     if (!obj)
       return -1;
     exc_descriptors[i].cls = obj;
@@ -219,13 +213,15 @@ static void make_exception(int res, sqlite3 *db)
 {
   int i;
   const char *errmsg = NULL;
+  int error_offset = -1;
 
   if (db)
     errmsg = apsw_get_errmsg();
   if (!errmsg)
     errmsg = "error";
 
-  APSW_FAULT_INJECT(UnknownSQLiteErrorCode, , res = 0xfe);
+  if (db)
+    _PYSQLITE_CALL_V(error_offset = sqlite3_error_offset(db));
 
   for (i = 0; exc_descriptors[i].name; i++)
     if (exc_descriptors[i].code == (res & 0xff))
@@ -235,12 +231,33 @@ static void make_exception(int res, sqlite3 *db)
       PyErr_Format(exc_descriptors[i].cls, "%sError: %s", exc_descriptors[i].name, errmsg);
       PyErr_Fetch(&etype, &eval, &etb);
       PyErr_NormalizeException(&etype, &eval, &etb);
+
+      assert(!PyErr_Occurred());
       tmp = PyLong_FromLongLong(res & 0xff);
-      PyObject_SetAttrString(eval, "result", tmp);
+      if (!tmp)
+        goto error;
+
+      if (PyObject_SetAttrString(eval, "result", tmp))
+        goto error;
+
       Py_DECREF(tmp);
       tmp = PyLong_FromLongLong(res);
-      PyObject_SetAttrString(eval, "extendedresult", tmp);
+      if (!tmp)
+        goto error;
+
+      if (PyObject_SetAttrString(eval, "extendedresult", tmp))
+        goto error;
       Py_DECREF(tmp);
+
+      tmp = PyLong_FromLong(error_offset);
+      if (!tmp)
+        goto error;
+
+      PyObject_SetAttrString(eval, "error_offset", tmp);
+    error:
+      Py_XDECREF(tmp);
+      if (PyErr_Occurred())
+        apsw_write_unraisable(NULL);
       PyErr_Restore(etype, eval, etb);
       assert(PyErr_Occurred());
       return;
@@ -279,9 +296,13 @@ MakeSqliteMsgFromPyException(char **errmsg)
         /* extract it */
         PyObject *extended = PyObject_GetAttrString(evalue, "extendedresult");
         if (extended && PyLong_Check(extended))
-          res = PyLong_AsLong(extended);
+          res = PyLong_AsInt(extended);
         Py_XDECREF(extended);
+        PyErr_Clear();
       }
+      /* this can happen with inopportune failures in the above */
+      if (res < 1)
+        res = SQLITE_ERROR;
       break;
     }
 
@@ -291,17 +312,27 @@ MakeSqliteMsgFromPyException(char **errmsg)
     if (!str && evalue)
       str = PyObject_Str(evalue);
     if (!str && etype)
+    {
+      PyErr_Clear();
       str = PyObject_Str(etype);
+    }
     if (!str)
+    {
+      PyErr_Clear();
       str = PyUnicode_FromString("python exception with no information");
-    if (*errmsg)
+    }
+    if (*errmsg && str)
+    {
       sqlite3_free(*errmsg);
-    *errmsg = sqlite3_mprintf("%s", PyBytes_AsString(str));
+      *errmsg = sqlite3_mprintf("%s", PyUnicode_AsUTF8(str));
+    }
 
     Py_XDECREF(str);
   }
 
   PyErr_Restore(etype, evalue, etraceback);
   assert(PyErr_Occurred());
+  assert(res != -1);
+  assert(res > 0);
   return res;
 }
