@@ -119,6 +119,126 @@ across.
 
 */
 
+/* make working with file control pragma easier */
+
+/** .. class:: VFSFcntlPragma
+
+A helper class to work with `SQLITE_FCNTL_PRAGMA
+<https://sqlite.org/c3ref/c_fcntl_begin_atomic_write.html#sqlitefcntlpragma>`__
+in :meth:`VFSFile.xFileControl`. The :ref:`example <example_vfs>`
+shows usage of this class.
+
+It is only valid while in :meth:`VFSFile.xFileControl`, and using
+outside of that will result in memory corruption and crashes.
+
+*/
+typedef struct apswfcntl_pragma
+{
+  PyObject_HEAD char **strings;
+} apswfcntl_pragma;
+
+static PyObject *
+apswfcntl_pragma_new(PyTypeObject *type, PyObject *Py_UNUSED(args), PyObject *Py_UNUSED(kwds))
+{
+  apswfcntl_pragma *self = (apswfcntl_pragma *)type->tp_alloc(type, 0);
+  if (self != NULL)
+  {
+    self->strings = NULL;
+  }
+  return (PyObject *)self;
+}
+
+/** .. method:: __init__(pointer: int)
+
+The pointer must be what your xFileControl method received.
+
+*/
+static int
+apswfcntl_pragma_init(apswfcntl_pragma *self, PyObject *args, PyObject *kwds)
+{
+  void *pointer = NULL;
+  {
+    static char *kwlist[] = {"pointer", NULL};
+    VFSFcntlPragma_init_CHECK;
+    argcheck_pointer_param pointer_param = {&pointer, VFSFcntlPragma_init_pointer_MSG};
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "O&:" VFSFcntlPragma_init_USAGE, kwlist, argcheck_pointer, &pointer_param))
+      return -1;
+  }
+  self->strings = pointer;
+  return 0;
+}
+
+/** .. attribute:: result
+    :type: str | None
+
+    The first element which becomes the result or error message
+*/
+static PyObject *
+apswfcntl_pragma_get_result(apswfcntl_pragma *self)
+{
+  return convertutf8string(self->strings[0]);
+}
+
+static int
+apswfcntl_pragma_set_result(apswfcntl_pragma *self, PyObject *value)
+{
+  if (!Py_IsNone(value) && !PyUnicode_Check(value))
+  {
+    PyErr_Format(PyExc_TypeError, "Expected None or str, not %s", Py_TypeName(value));
+    return -1;
+  }
+  if (self->strings[0])
+  {
+    sqlite3_free(self->strings[0]);
+    self->strings[0] = NULL;
+  }
+  if (!Py_IsNone(value))
+    self->strings[0] = sqlite3_mprintf("%s", PyUnicode_AsUTF8(value));
+  return 0;
+}
+
+/** .. attribute:: name
+    :type: str
+
+    The name of the pragma
+*/
+static PyObject *
+apswfcntl_pragma_get_name(apswfcntl_pragma *self)
+{
+  return convertutf8string(self->strings[1]);
+}
+
+/** .. attribute:: value
+    :type: str | None
+
+    The value for the pragma, if provided else None,
+*/
+static PyObject *
+apswfcntl_pragma_get_value(apswfcntl_pragma *self)
+{
+  return convertutf8string(self->strings[2]);
+}
+
+static PyGetSetDef apswfcntl_pragma_getsetters[] = {
+    {"result", (getter)apswfcntl_pragma_get_result, (setter)apswfcntl_pragma_set_result, VFSFcntlPragma_result_DOC},
+    {"name", (getter)apswfcntl_pragma_get_name, NULL, VFSFcntlPragma_name_DOC},
+    {"value", (getter)apswfcntl_pragma_get_value, NULL, VFSFcntlPragma_value_DOC},
+    /* sentinel */
+    {
+        NULL, NULL, NULL, NULL}};
+
+static PyTypeObject apswfcntl_pragma_Type = {
+    PyVarObject_HEAD_INIT(NULL, 0)
+        .tp_name = "apsw.VFSFcntlPragma",
+    .tp_doc = VFSFcntlPragma_class_DOC,
+    .tp_basicsize = sizeof(apswfcntl_pragma),
+    .tp_itemsize = 0,
+    .tp_flags = Py_TPFLAGS_DEFAULT,
+    .tp_new = apswfcntl_pragma_new,
+    .tp_init = (initproc)apswfcntl_pragma_init,
+    .tp_getset = apswfcntl_pragma_getsetters,
+};
+
 /* Naming convention prefixes.  Since sqlite3.c is #included alongside
    this file we have to ensure there is no clash with its names.
    There are two objects - the VFS itself and a VFSFile as returned
@@ -1133,10 +1253,10 @@ apswvfs_xCurrentTime(sqlite3_vfs *vfs, double *julian)
 /** .. method:: xCurrentTime()  -> float
 
   Return the `Julian Day Number
-  <http://en.wikipedia.org/wiki/Julian_day>`_ as a floating point
+  <https://en.wikipedia.org/wiki/Julian_day>`_ as a floating point
   number where the integer portion is the day and the fractional part
   is the time. Do not adjust for timezone (ie use `UTC
-  <http://en.wikipedia.org/wiki/Universal_Time>`_).
+  <https://en.wikipedia.org/wiki/Universal_Time>`_).
 */
 static PyObject *
 apswvfspy_xCurrentTime(APSWVFS *self)
@@ -1185,7 +1305,7 @@ apswvfs_xCurrentTimeInt64(sqlite3_vfs *vfs, sqlite3_int64 *time)
 /** .. method:: xCurrentTimeInt64()  -> int
 
   Returns as an integer the `Julian Day Number
-  <http://en.wikipedia.org/wiki/Julian_day>`__ multiplied by 86400000
+  <https://en.wikipedia.org/wiki/Julian_day>`__ multiplied by 86400000
   (the number of milliseconds in a 24-hour day).
 */
 static PyObject *
@@ -1822,9 +1942,10 @@ static PyTypeObject APSWVFSType =
 /** .. method:: excepthook(etype: type[BaseException], evalue: BaseException, etraceback: Optional[types.TracebackType]) ->None
 
     Called when there has been an exception in a :class:`VFSFile`
-    routine.  The default implementation calls ``sys.excepthook`` and
-    if that fails then ``PyErr_Display``.  The three arguments
-    correspond to what ``sys.exc_info()`` would return.
+    routine.  The default implementation passes the exception information
+    to sqlite3_log, and the first non-error of
+    :func:`sys.unraisablehook` and :func:`sys.excepthook`, falling back to
+    `PyErr_Display`.
 
     :param etype: The exception type
     :param evalue: The exception  value
@@ -2035,7 +2156,7 @@ apswvfsfile_xRead(sqlite3_file *file, void *bufout, int amount, sqlite3_int64 of
   if (py3buffer.len < amount)
   {
     result = SQLITE_IOERR_SHORT_READ;
-    memset(bufout, 0, amount); /* see https://sqlite.org/cvstrac/chngview?cn=5867 */
+    memset(bufout, 0, amount);
     memcpy(bufout, py3buffer.buf, py3buffer.len);
   }
   else
@@ -2097,8 +2218,7 @@ apswvfsfilepy_xRead(APSWVFSFile *self, PyObject *args, PyObject *kwds)
   if (res == SQLITE_IOERR_SHORT_READ)
   {
     /* We don't know how short the read was, so look for first
-         non-trailing null byte.  See
-         https://sqlite.org/cvstrac/chngview?cn=5867 */
+         non-trailing null byte.  */
     while (amount && PyBytes_AS_STRING(buffy)[amount - 1] == 0)
       amount--;
     if (_PyBytes_Resize(&buffy, amount))
@@ -2652,7 +2772,7 @@ apswvfsfile_xFileControl(sqlite3_file *file, int op, void *pArg)
                     obj=ctypes.py_object.from_address(ptr).value
                 else:
                     # this ensures superclass implementation is called
-                    return super(MyFile, self).xFileControl(op, ptr)
+                    return super().xFileControl(op, ptr)
     # we understood the op
           return True
 */
